@@ -1,6 +1,7 @@
 context('GIS tools')
 
 test_that('gh_to_sp works', {
+  skip_if(!requireNamespace('sp'), "sp installation required")
   mauritius = c("mk2u", "mk2e", "mk2g", "mk35", "mk3h",
                 "mk3j", "mk2v", "mk2t", "mk2s")
 
@@ -14,12 +15,7 @@ test_that('gh_to_sp works', {
                         57.3046875, -20.390625, -20.21484375,
                         -20.21484375, -20.390625, -20.390625),
                       nrow = 5L, ncol = 2L))
-  if (!requireNamespace('sp')) {
-    skip("sp installation required for testing CRS equality")
-  } else {
-    expect_equal(ghSP@proj4string,
-                 sp::CRS("+init=epsg:4326"))
-  }
+  expect_equal(ghSP@proj4string, sp::CRS("+init=epsg:4326"))
 
   # duplicate inputs dropped
   expect_warning(ghSP2 <- gh_to_sp(rep(mauritius, 2L)),
@@ -28,6 +24,7 @@ test_that('gh_to_sp works', {
 })
 
 test_that('gh_to_spdf.default works', {
+  skip_if(!requireNamespace('sp'), "sp installation required")
   urumqi = c("tzy3", "tzy0", "tzy2", "tzy8", "tzy9",
              "tzyd", "tzy6", "tzy4", "tzy1")
 
@@ -41,12 +38,8 @@ test_that('gh_to_spdf.default works', {
                         87.890625, 87.5390625, 43.59375, 43.76953125,
                         43.76953125, 43.59375, 43.59375),
                       nrow = 5L, ncol = 2L))
-  if (!requireNamespace('sp')) {
-    skip("sp installation required for testing CRS equality")
-  } else {
-    expect_equal(ghSPDF@proj4string,
-                 sp::CRS("+init=epsg:4326"))
-  }
+  expect_equal(ghSPDF@proj4string, sp::CRS("+init=epsg:4326"))
+
   DF = data.frame(ID = 1:9, row.names = urumqi)
   expect_equal(ghSPDF@data, DF)
   # check also duplicated input (#11)
@@ -56,6 +49,7 @@ test_that('gh_to_spdf.default works', {
 })
 
 test_that('gh_to_spdf.data.frame works', {
+  skip_if(!requireNamespace('sp'), "sp installation required")
   urumqi = c("tzy3", "tzy0", "tzy2", "tzy8", "tzy9",
              "tzyd", "tzy6", "tzy4", "tzy1")
   DF = data.frame(
@@ -72,12 +66,7 @@ test_that('gh_to_spdf.data.frame works', {
                         87.890625, 87.5390625, 43.59375, 43.76953125,
                         43.76953125, 43.59375, 43.59375),
                       nrow = 5L, ncol = 2L))
-  if (!requireNamespace('sp')) {
-    skip("sp installation required for testing CRS equality")
-  } else {
-    expect_equal(ghSPDF@proj4string,
-                 sp::CRS("+init=epsg:4326"))
-  }
+  expect_equal(ghSPDF@proj4string, sp::CRS("+init=epsg:4326"))
   expect_equal(ghSPDF@data, DF)
 
   # duplicated inputs (#11)
@@ -96,6 +85,7 @@ test_that('gh_to_spdf.data.frame works', {
 })
 
 test_that('gh_to_sf works', {
+  skip_if(!requireNamespace('sf'), "sf installation required")
   baku = c("tp5my", "tp5mt", "tp5mw", "tp5mx", "tp5mz",
            "tp5qp", "tp5qn", "tp5qj", "tp5mv")
 
@@ -115,6 +105,7 @@ test_that('gh_to_sf works', {
 })
 
 test_that('gh_covering works', {
+  skip_if(!requireNamespace('sp'), "sp installation required")
   banjarmasin = sp::SpatialPoints(cbind(
     c(114.605, 114.5716, 114.627, 114.5922, 114.6321,
       114.5804, 114.6046, 114.6028, 114.6232, 114.5792),
@@ -124,7 +115,8 @@ test_that('gh_covering works', {
 
   # core
   banjarmasin_cover = gh_covering(banjarmasin)
-  sp::proj4string(banjarmasin) = sp::CRS("+init=epsg:4326")
+  wgs = sp::CRS("+init=epsg:4326")
+  sp::proj4string(banjarmasin) = wgs
   # use gUnaryUnion to overcome rgeos bug as reported 2019-08-16
   expect_true(!any(is.na(sp::over(banjarmasin, banjarmasin_cover))))
   expect_equal(sort(rownames(banjarmasin_cover@data))[1:10],
@@ -139,6 +131,45 @@ test_that('gh_covering works', {
                c("qx3kzm", "qx3kzx", "qx3mp3", "qx3mpb", "qx3mpu",
                  "qx3mpz", "qx3mr5", "qx3sbt", "qx3t06", "qx3t22"))
   expect_length(banjarmasin_tight, 10L)
+  # #27 -- proj4string<- doesn't mutate object, but proj4string() <- does?
+  sp::proj4string(banjarmasin) = NA_character_
+  expect_identical(
+    banjarmasin_tight,
+    sp::`proj4string<-`(gh_covering(banjarmasin, minimal = TRUE), wgs)
+  )
+
+  # errors
+  expect_error(gh_covering(4L), 'Object to cover must be Spatial', fixed = TRUE)
+})
+
+test_that('gh_covering_sf works', {
+  skip_if(!requireNamespace('sp'), "sp installation required")
+  skip_if(!requireNamespace('sf'), "sp installation required")
+  banjarmasin = sf::st_as_sf(sp::SpatialPoints(cbind(
+    c(114.605, 114.5716, 114.627, 114.5922, 114.6321,
+      114.5804, 114.6046, 114.6028, 114.6232, 114.5792),
+    c(-3.3346, -3.2746, -3.2948, -3.3424, -3.3523,
+      -3.3304, -3.3005, -3.3141, -3.326, -3.3552)
+  )))
+
+  # core
+  banjarmasin_cover = gh_covering(banjarmasin)
+  sf::st_crs(banjarmasin) = sf::st_crs("+init=epsg:4326")
+  # use gUnaryUnion to overcome rgeos bug as reported 2019-08-16
+  expect_true(!any(is.na(sapply(sf::st_intersects(banjarmasin,banjarmasin_cover),
+                                function(z) if (length(z)==0) NA_integer_ else z[1]))))
+  expect_equal(sort(rownames(banjarmasin_cover))[1:10],
+               c("qx3kzj", "qx3kzm", "qx3kzn", "qx3kzp", "qx3kzq",
+                 "qx3kzr", "qx3kzt", "qx3kzv", "qx3kzw", "qx3kzx"))
+  expect_length(banjarmasin_cover$geometry, 112L)
+
+  # arguments
+  expect_equal(nrow(gh_covering(banjarmasin, 5L)), 9L)
+  banjarmasin_tight = gh_covering(banjarmasin, minimal = TRUE)
+  expect_equal(sort(rownames(banjarmasin_tight))[1:10],
+               c("qx3kzm", "qx3kzx", "qx3mp3", "qx3mpb", "qx3mpu",
+                 "qx3mpz", "qx3mr5", "qx3sbt", "qx3t06", "qx3t22"))
+  expect_equal(nrow(banjarmasin_tight), 10L)
 
   # errors
   expect_error(gh_covering(4L), 'Object to cover must be Spatial', fixed = TRUE)
